==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/p_sys_execveat/p_sys_execveat.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept execveat syscall
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When process calls execve, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 02.I.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"

#ifdef P_LKRG_EXPLOIT_DETECTION_SYS_EXECVEAT_H

char p_sys_execveat_kretprobe_state = 0x0;

static struct kretprobe p_sys_execveat_kretprobe = {
    .kp.symbol_name = P_GET_SYSCALL_NAME(execveat),
    .handler = p_sys_execveat_ret,
    .entry_handler = p_sys_execveat_entry,
    .data_size = sizeof(struct p_sys_execveat_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_execveat_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_sys_execveat_entry>\n");

//   p_ed_enforce_validation();

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_debug_kprobe_log(
          "Leaving function <p_sys_execveat_entry>\n");

   return 0;
}


int p_sys_execveat_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_sys_execveat_ret>\n");
/*
   p_inode = p_get_inode_from_task(current);

   p_debug_kprobe_log(
//   p_print_log(P_LKRG_CRIT,
          "p_sys_execveat_ret: returned value => %ld comm[%s] Pid:%d inode[%ld]\n",
           p_regs->ax,current->comm,current->pid,p_inode->i_ino);
*/

   // Update process
   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
      p_update_ed_process(p_tmp, current);
      p_set_ed_process_on(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_sys_execveat_ret>\n");

   return 0x0;
}


int p_install_sys_execveat_hook(void) {

   int p_ret;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_sys_execveat_hook>\n");

   if ( (p_ret = register_kretprobe(&p_sys_execveat_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_sys_execveat_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_uninstall_sys_execveat_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_sys_execveat_kretprobe.kp.symbol_name,
               p_sys_execveat_kretprobe.kp.addr);
   p_sys_execveat_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_uninstall_sys_execveat_hook_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_sys_execveat_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_sys_execveat_hook(void) {

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_sys_execveat_hook>\n");

   if (!p_sys_execveat_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_sys_execveat_kretprobe.kp.symbol_name,
                  p_sys_execveat_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_sys_execveat_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_sys_execveat_kretprobe.kp.symbol_name,
                  p_sys_execveat_kretprobe.kp.addr,
                  p_sys_execveat_kretprobe.nmissed);
      p_sys_execveat_kretprobe_state = 0x0;
   }

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_sys_execveat_hook>\n");
}

#endif

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/p_sys_execveat/p_sys_execveat.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept execveat syscall
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When process calls execve, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 02.I.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"

#ifdef P_LKRG_EXPLOIT_DETECTION_SYS_EXECVEAT_H

char p_sys_execveat_kretprobe_state = 0x0;

static struct kretprobe p_sys_execveat_kretprobe = {
    .kp.symbol_name = P_GET_SYSCALL_NAME(execveat),
    .handler = p_sys_execveat_ret,
    .entry_handler = p_sys_execveat_entry,
    .data_size = sizeof(struct p_sys_execveat_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_execveat_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_execveat_entry>\n");

//   p_ed_enforce_validation();

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   p_debug_kprobe_log(
          "Leaving function <p_sys_execveat_entry>\n");

   return 0;
}


int p_sys_execveat_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_execveat_ret>\n");
/*
   p_inode = p_get_inode_from_task(current);

   p_debug_kprobe_log(
//   p_print_log(P_LKRG_CRIT,
          "p_sys_execveat_ret: returned value => %ld comm[%s] Pid:%d inode[%ld]\n",
           p_regs_get_ret(p_regs),current->comm,current->pid,p_inode->i_ino);
*/

   // Update process
   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      if (!p_regs_get_ret(p_regs) ||
          (!task_pid_nr(p_tmp->p_ed_task.p_task) && !p_tmp->p_ed_task.p_task->comm[0])) {
         // This process is on the ED list - update information!
         p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
         p_update_ed_process(p_tmp, current, 1);
         p_reset_ed_flags(p_tmp);
      } else {
         p_set_ed_process_on(p_tmp);
      }
   }
   p_tasks_write_unlock(&p_flags);

   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_sys_execveat_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(sys_execveat)

#endif
